@@ -40,7 +40,6 @@
 /* Global Variable definitions                                          */
 /*----------------------------------------------------------------------*/
 
-extern short textpos, textend;  /* keep track of the cursor position in text */
 extern Display  *dpy;    /* Works well to make this globally accessible */
 extern int *appcolors;
 extern Cursor   appcursors[NUM_CURSORS];
@@ -456,7 +455,7 @@ void textbutton(u_char dopin, int x, int y)
    UDrawTLine(*newlabel);
    areawin->origin.x = userpt.x;
    areawin->origin.y = userpt.y;
-   textpos = 1;  /* Text position is *after* the font declaration */
+   areawin->textpos = 1;  /* Text position is *after* the font declaration */
 }
 
 /*----------------------------------------------------------------------*/
@@ -471,10 +470,10 @@ void charreport(labelptr curlabel)
    stringpart *strptr;
 
    _STR2[0] = '\0';
-   for (i = textpos - MAXCHARS; i <= textpos + MAXCHARS - 1; i++) {
+   for (i = areawin->textpos - MAXCHARS; i <= areawin->textpos + MAXCHARS - 1; i++) {
       if (i < 0) continue; 
       strptr = findstringpart(i, &locpos, curlabel->string, areawin->topinstance);
-      if (i == textpos) {
+      if (i == areawin->textpos) {
 	 strncat(_STR2, "| ", cleft);
 	 cleft -= 2;
       }
@@ -528,25 +527,27 @@ Boolean labeltext(int keypressed, char *clientdata)
 
    curlabel = TOLABEL(EDITPART);
 
-   if (curlabel == NULL || curlabel->type != LABEL || textpos <= 0) {
+   if (curlabel == NULL || curlabel->type != LABEL || areawin->textpos <= 0) {
       Wprintf("Error:  Bad label string");
+      eventmode = NORMAL_MODE;
       return FALSE;
    }
 
    /* find text segment of the current position */
-   curpos = findstringpart(textpos, &locpos, curlabel->string, areawin->topinstance);
+   curpos = findstringpart(areawin->textpos, &locpos, curlabel->string,
+		areawin->topinstance);
 
    UDrawTLine(curlabel);
 
    if (clientdata != NULL && keypressed == TEXT_DELETE) {
-      if (textpos > 1) {
+      if (areawin->textpos > 1) {
          int curloc, strpos;
          stringpart *strptr;
 
-         if (textend == 0) textend = textpos - 1;
+         if (areawin->textend == 0) areawin->textend = areawin->textpos - 1;
 
          undrawtext(curlabel);
-         for (strpos = textpos - 1; strpos >= textend; strpos--) {
+         for (strpos = areawin->textpos - 1; strpos >= areawin->textend; strpos--) {
 	    strptr = findstringpart(strpos, &curloc, curlabel->string,
 			 areawin->topinstance);
 	    if (curloc >= 0) {
@@ -564,22 +565,22 @@ Boolean labeltext(int keypressed, char *clientdata)
 	       if ((strptr->type != PARAM_START) && (strptr->type != PARAM_END))
 	          deletestring(strptr, &curlabel->string, areawin->topinstance);
 	       else
-	          textpos++;
+	          areawin->textpos++;
 	    }
 	    else
 	       Fprintf(stdout, "Error:  Unexpected NULL string part\n");
-            textpos--;
+            areawin->textpos--;
          }
-         textend = 0;
+         areawin->textend = 0;
          do_redraw = True;
       }
    }
    else if (clientdata != NULL && keypressed == TEXT_DEL_PARAM) {
-      if (textpos > 1) {
+      if (areawin->textpos > 1) {
          int curloc, strpos;
          stringpart *strptr;
 
-         strptr = findstringpart(textpos - 1, &curloc, curlabel->string,
+         strptr = findstringpart(areawin->textpos - 1, &curloc, curlabel->string,
 			 areawin->topinstance);
          if ((curloc < 0) && (strptr != NULL) && (strptr->type == PARAM_END)) {
 	    undrawtext(curlabel);
@@ -671,7 +672,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 	       undrawtext(curlabel);
 	       curlabel->string->nextpart->data.string = (char *)realloc(
 			curlabel->string->nextpart->data.string,
-		  	(strlen(_STR) + 1) * sizeof(char));
+		  	(strlen(libobj->name) + 1) * sizeof(char));
 	       strcpy(curlabel->string->nextpart->data.string, libobj->name);
                XcSetFunction(GXcopy);
 	       redrawtext(curlabel);
@@ -737,15 +738,15 @@ Boolean labeltext(int keypressed, char *clientdata)
       return r;
    }
    else if (clientdata != NULL && keypressed == TEXT_RIGHT) {
-      if (curpos != NULL) textpos++;
+      if (curpos != NULL) areawin->textpos++;
    }
    else if (clientdata != NULL && keypressed == TEXT_LEFT) {
-      if (textpos > 1) textpos--;
+      if (areawin->textpos > 1) areawin->textpos--;
    }
    else if (clientdata != NULL && keypressed == TEXT_DOWN) {
       while (curpos != NULL) {
-	 textpos++;
-	 curpos = findstringpart(textpos, &locpos, curlabel->string,
+	 areawin->textpos++;
+	 curpos = findstringpart(areawin->textpos, &locpos, curlabel->string,
 			areawin->topinstance);
 	 if (curpos != NULL)
 	    if (curpos->type == RETURN)
@@ -753,36 +754,38 @@ Boolean labeltext(int keypressed, char *clientdata)
       }
    }
    else if (clientdata != NULL && keypressed == TEXT_UP) {
-      while (textpos > 1) {
-	 textpos--;
-	 curpos = findstringpart(textpos, &locpos, curlabel->string,
+      while (areawin->textpos > 1) {
+	 areawin->textpos--;
+	 curpos = findstringpart(areawin->textpos, &locpos, curlabel->string,
 	 		areawin->topinstance);
 	 if (curpos->type == RETURN) {
-	    if (textpos > 1) textpos--;
+	    if (areawin->textpos > 1) areawin->textpos--;
 	    break;
 	 }
       }
    }
    else if (clientdata != NULL && keypressed == TEXT_HOME)
-      textpos = 1;
+      areawin->textpos = 1;
    else if (clientdata != NULL && keypressed == TEXT_END)
-      textpos = stringlength(curlabel->string, True, areawin->topinstance);
+      areawin->textpos = stringlength(curlabel->string, True, areawin->topinstance);
    else if (clientdata != NULL && keypressed == TEXT_SPLIT) {
       labelptr *newlabel;
       XPoint points[4], points1[4], points2[4];
 
       /* Everything after the cursor gets dumped into a new label */
 
-      if ((textpos > 1) && (curpos != NULL)) {
+      if ((areawin->textpos > 1) && (curpos != NULL)) {
 	 labelbbox(curlabel, points, areawin->topinstance);
          undrawtext(curlabel);
 	 NEW_LABEL(newlabel, topobject);
 	 labeldefaults(*newlabel, curlabel->pin, curlabel->position.x,
 		curlabel->position.y);
          if (locpos > 0)
-            curpos = splitstring(textpos, &curlabel->string, areawin->topinstance);
+            curpos = splitstring(areawin->textpos, &curlabel->string,
+			areawin->topinstance);
 	 /* move back one position to find end of top part of string */
-         curpos = splitstring(textpos - 1, &curlabel->string, areawin->topinstance);
+         curpos = splitstring(areawin->textpos - 1, &curlabel->string,
+			areawin->topinstance);
 	 if (curpos->nextpart->type == FONT_NAME) {
 	    freelabel((*newlabel)->string);
 	    (*newlabel)->string = curpos->nextpart;
@@ -820,7 +823,8 @@ Boolean labeltext(int keypressed, char *clientdata)
       /* erase first before redrawing unless the string is empty */
       undrawtext(curlabel);
       if (locpos > 0) {
-         curpos = splitstring(textpos, &curlabel->string, areawin->topinstance);
+         curpos = splitstring(areawin->textpos, &curlabel->string,
+			areawin->topinstance);
 	 curpos = curpos->nextpart;
       }
       newpart = makesegment(&curlabel->string, curpos);
@@ -849,8 +853,8 @@ Boolean labeltext(int keypressed, char *clientdata)
 	    else {
 	       /* Forward edit cursor position to the end of the parameter */
 	       do {
-	          textpos++;
-	          curpos = findstringpart(textpos, &locpos, curlabel->string,
+	          areawin->textpos++;
+	          curpos = findstringpart(areawin->textpos, &locpos, curlabel->string,
 	 		areawin->topinstance);
 	       } while (curpos->type != PARAM_END);
 	    }
@@ -862,7 +866,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 	 r = FALSE;
       }
       else {
-         textpos++;
+         areawin->textpos++;
       }
       do_redraw = True;
    }
@@ -879,8 +883,8 @@ Boolean labeltext(int keypressed, char *clientdata)
       /* Current position is not in a text string */
       if (locpos < 0) {
 
-         /* Find part of string which is immediately in front of textpos */
-         lastpos = findstringpart(textpos - 1, &locpos, curlabel->string,
+         /* Find part of string which is immediately in front of areawin->textpos */
+         lastpos = findstringpart(areawin->textpos - 1, &locpos, curlabel->string,
 		areawin->topinstance);
 
 	 /* No text on either side to attach to: make a new text segment */
@@ -906,7 +910,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 		strlen(curpos->data.string + locpos) + 1);
          *(curpos->data.string + locpos) = keypressed;
       }
-      textpos++;	/* move forward to next text position */
+      areawin->textpos++;	/* move forward to next text position */
       do_redraw = True;
       r = TRUE;
    }
@@ -917,7 +921,7 @@ Boolean labeltext(int keypressed, char *clientdata)
       short beglength;
 
       tmpext = ULength(curlabel, areawin->topinstance, curlabel->scale,
-		textpos, NULL);
+		areawin->textpos, NULL);
       beglength = tmpext.width;
 	
       tmpext = ULength(curlabel, areawin->topinstance, curlabel->scale,
@@ -947,7 +951,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 
       /* find current font and adjust menubuttons as necessary */
 
-      cfont = findcurfont(textpos, curlabel->string, areawin->topinstance);
+      cfont = findcurfont(areawin->textpos, curlabel->string, areawin->topinstance);
       if (cfont < 0) {
          Wprintf("Error:  Illegal label string");
          return r;
@@ -955,7 +959,7 @@ Boolean labeltext(int keypressed, char *clientdata)
       else
          setfontmarks(cfont, -1);
 
-      textend = 0;
+      areawin->textend = 0;
    }
    return r;
 }
@@ -1100,6 +1104,32 @@ genericptr getsubpart(pathptr editpath)
    return NULL;
 }
 
+/*----------------------------------------------------------------------*/
+/* Same as the above routine, but returns a part index			*/
+/*----------------------------------------------------------------------*/
+
+int getsubpartindex(pathptr editpath)
+{
+   genericptr *pgen;
+   int ridx = 0;
+
+   for (pgen = editpath->plist; pgen < editpath->plist + editpath->parts; pgen++) {
+      switch (ELEMENTTYPE(*pgen)) {
+	 case POLYGON:
+	    if ((TOPOLY(pgen))->cycle >= 0) return ridx;
+	    break;
+	 case ARC:
+	    if ((TOARC(pgen))->cycle >= 0) return ridx;
+	    break;
+	 case SPLINE:
+	    if ((TOSPLINE(pgen))->cycle >= 0) return ridx;
+	    break;
+      }
+      ridx++;
+   }
+   return -1;
+}
+
 /*----------------------------------------------------------------------*/
 /* Adjust path points for parts connected to points that have been	*/
 /* adjusted during an edit.						*/
