@@ -1343,9 +1343,10 @@ void catmove(int x, int y)
    drawarea(NULL, NULL, NULL);
 }
 
-/*--------------------------------------------------------*/
-/* Make a duplicate of an object, put in the User Library */
-/*--------------------------------------------------------*/
+/*------------------------------------------------------*/
+/* Make a duplicate of an object, put in the User	*/
+/* Library or the current library (if we're in one).	*/
+/*------------------------------------------------------*/
 
 void copycat()
 {
@@ -1353,7 +1354,10 @@ void copycat()
    objectptr *newobj, *curlib, oldobj;
    objinstptr libobj;
    oparamptr ops, newops;
-   int i, libnum = USERLIB - LIBRARY ;
+   int i, libnum;
+
+   libnum = is_library(topobject);
+   if (libnum < 0) libnum = USERLIB - LIBRARY;  /* default */
 
    for (newselect = areawin->selectlist; newselect < areawin->selectlist
 	  + areawin->selects; newselect++) {
@@ -1553,6 +1557,7 @@ void catalog_op(int op, int x, int y)
 		  eventmode = NORMAL_MODE;
 	       }
 	       else if ((op == XCF_Library_Pop) || (op == XCF_Library_Copy)) {
+		  int saveselects;
 
 	          /* revert to old page */
 
@@ -1567,7 +1572,12 @@ void catalog_op(int op, int x, int y)
 
 	          snap(x, y, &oldpos);
 
+		  /* setpage() will make a call to updateparams, which can	*/
+		  /* cause a crash if we have selections.			*/
+		  saveselects = areawin->selects;
+		  areawin->selects = 0;
    	          setpage(FALSE);
+		  areawin->selects = saveselects;
 	    
 	          snap(x, y, &areawin->save);
 	          xdiff = areawin->save.x - oldpos.x;
