@@ -53,7 +53,6 @@ extern ApplicationData appdata;
 extern Cursor	  appcursors[NUM_CURSORS];
 extern fontinfo *fonts;
 extern short fontcount;
-extern short textpos, textend;
 
 #ifdef TCL_WRAPPER
 extern Tcl_Interp *xcinterp;
@@ -188,12 +187,13 @@ void changetextscale(float newscale)
 
    if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       settext = *((labelptr *)EDITPART);
-      if (textpos > 0 || textpos < stringlength(settext->string, True,
+      if (areawin->textpos > 0 || areawin->textpos < stringlength(settext->string, True,
 		areawin->topinstance)) {
 	 undrawtext(settext);
-	 strptr = findstringpart(textpos - 1, NULL, settext->string,
+	 strptr = findstringpart(areawin->textpos - 1, NULL, settext->string,
+			areawin->topinstance);
+	 nextptr = findstringpart(areawin->textpos, NULL, settext->string,
 			areawin->topinstance);
-	 nextptr = findstringpart(textpos, NULL, settext->string, areawin->topinstance);
 	 if (strptr->type == FONT_SCALE)
 	    strptr->data.scale = newscale;
 	 else if (nextptr && nextptr->type == FONT_SCALE)
@@ -384,12 +384,12 @@ labelptr gettextsize(float **floatptr)
    if (floatptr) *floatptr = &areawin->textscale;
 
    if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
-      if (textpos > 0 || textpos < stringlength(settext->string, True,
-		areawin->topinstance)) {
+      if (areawin->textpos > 0 || areawin->textpos < stringlength(settext->string,
+		True, areawin->topinstance)) {
          settext = *((labelptr *)EDITPART);
-	 strptr = findstringpart(textpos - 1, NULL, settext->string,
+	 strptr = findstringpart(areawin->textpos - 1, NULL, settext->string,
 			areawin->topinstance);
-	 nextptr = findstringpart(textpos, NULL, settext->string,
+	 nextptr = findstringpart(areawin->textpos, NULL, settext->string,
 			areawin->topinstance);
 	 if (strptr->type == FONT_SCALE) {
 	    if (floatptr) *floatptr = &strptr->data.scale;
@@ -1077,8 +1077,10 @@ void setcolor(xcWidget w, int cindex)
 
    if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       labelptr curlabel = TOLABEL(EDITPART);
-      strptr = findstringpart(textpos - 1, NULL, curlabel->string, areawin->topinstance);
-      nextptr = findstringpart(textpos, NULL, curlabel->string, areawin->topinstance);
+      strptr = findstringpart(areawin->textpos - 1, NULL, curlabel->string,
+			areawin->topinstance);
+      nextptr = findstringpart(areawin->textpos, NULL, curlabel->string,
+			areawin->topinstance);
       if (strptr->type == FONT_COLOR) {
 	 undrawtext(curlabel);
 	 strptr->data.color = cindex;
@@ -1233,13 +1235,13 @@ void stringparam(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
    genericptr *settext;
 
-      if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
-         settext = (genericptr *)EDITPART;
-         makeparam(TOLABEL(settext), _STR2);
-         unselect_all();
-	 setparammarks(NULL);
-      }
-      else if (checkselect(LABEL)) parameterize(P_SUBSTRING, _STR2, -1);
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
+      settext = (genericptr *)EDITPART;
+      makeparam(TOLABEL(settext), _STR2);
+      unselect_all();
+      setparammarks(NULL);
+   }
+   else if (checkselect(LABEL)) parameterize(P_SUBSTRING, _STR2, -1);
 }
 
 /*--------------------------------------------------------------*/
@@ -1248,8 +1250,6 @@ void stringparam(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
 void startparam(xcWidget w, pointertype value, caddr_t calldata)
 {
-  /* genericptr *settext; (jdk) */
-
    if (value == (pointertype)P_SUBSTRING) {
       strcpy(_STR2, (calldata != NULL) ? (char *)calldata : "substring");
       stringparam(w, NULL, NULL);
@@ -1659,9 +1659,9 @@ void setfontval(xcWidget w, pointertype value, labelptr settext)
 
       /* if last byte was a font designator, use it */
 
-      if (textpos > 0 || textpos < stringlength(settext->string, True,
-			areawin->topinstance)) {
-	 strptr = findstringpart(textpos - 1, NULL, settext->string,
+      if (areawin->textpos > 0 || areawin->textpos < stringlength(settext->string,
+			True, areawin->topinstance)) {
+	 strptr = findstringpart(areawin->textpos - 1, NULL, settext->string,
 			areawin->topinstance);
 	 if (strptr->type == FONT_NAME) {
 	    tc = strptr->data.font;
@@ -1680,7 +1680,7 @@ void setfontval(xcWidget w, pointertype value, labelptr settext)
       }
 
       /* otherwise, look for the last style used in the string */
-      tc = findcurfont(textpos, settext->string, areawin->topinstance);
+      tc = findcurfont(areawin->textpos, settext->string, areawin->topinstance);
    }
    else tc = areawin->psfont;
 
@@ -1721,7 +1721,7 @@ void setfont(xcWidget w, pointertype value, caddr_t calldata)
       charreport(settext);
    }
    else {
-      textpos = 1;
+      areawin->textpos = 1;
       for (fselect = areawin->selectlist; fselect < areawin->selectlist +
 	    areawin->selects; fselect++) {
          if (SELECTTYPE(fselect) == LABEL) {
@@ -1750,8 +1750,10 @@ void setfontstyle(xcWidget w, pointertype value, labelptr settext)
 
       /* if last byte was a font designator, use it */
 
-      if (textpos > 0 || textpos < stringlength(settext->string, True, areawin->topinstance)) {
-	 strptr = findstringpart(textpos - 1, NULL, settext->string, areawin->topinstance);
+      if (areawin->textpos > 0 || areawin->textpos < stringlength(settext->string,
+			True, areawin->topinstance)) {
+	 strptr = findstringpart(areawin->textpos - 1, NULL, settext->string,
+			areawin->topinstance);
 	 if (strptr->type == FONT_NAME) {
 	    tc = strptr->data.font;
 
@@ -1774,7 +1776,7 @@ void setfontstyle(xcWidget w, pointertype value, labelptr settext)
       }
 
       /* otherwise, look for the last font used in the string */
-      tc = findcurfont(textpos - 2, settext->string, areawin->topinstance);
+      tc = findcurfont(areawin->textpos - 2, settext->string, areawin->topinstance);
    }
    else tc = areawin->psfont;
 
@@ -1815,7 +1817,7 @@ void fontstyle(xcWidget w, pointertype value, caddr_t nulldata)
       charreport(settext);
    }
    else {
-      textpos = 1;
+      areawin->textpos = 1;
       for (fselect = areawin->selectlist; fselect < areawin->selectlist +
 	    areawin->selects; fselect++) {
          if (SELECTTYPE(fselect) == LABEL) {
@@ -1844,9 +1846,9 @@ void setfontencoding(xcWidget w, pointertype value, labelptr settext)
 
       /* if last byte was a font designator, use it */
 
-      if (textpos > 0 || textpos < stringlength(settext->string, True,
-			areawin->topinstance)) {
-	 strptr = findstringpart(textpos - 1, NULL, settext->string,
+      if (areawin->textpos > 0 || areawin->textpos < stringlength(settext->string,
+		True, areawin->topinstance)) {
+	 strptr = findstringpart(areawin->textpos - 1, NULL, settext->string,
 			areawin->topinstance);
 	 if (strptr->type == FONT_NAME) {
 	    tc = strptr->data.font;
@@ -1870,7 +1872,7 @@ void setfontencoding(xcWidget w, pointertype value, labelptr settext)
       }
 
       /* otherwise, look for the last style used in the string */
-      tc = findcurfont(textpos - 2, settext->string, areawin->topinstance);
+      tc = findcurfont(areawin->textpos - 2, settext->string, areawin->topinstance);
    }
    else tc = areawin->psfont;
 
@@ -1910,7 +1912,7 @@ void fontencoding(xcWidget w, pointertype value, caddr_t nulldata)
       charreport(settext);
    }
    else {
-      textpos = 1;
+      areawin->textpos = 1;
       for (fselect = areawin->selectlist; fselect < areawin->selectlist +
 	    areawin->selects; fselect++) {
          if (SELECTTYPE(fselect) == LABEL) {
@@ -1941,7 +1943,7 @@ Boolean dospecial()
    curlabel = TOLABEL(EDITPART);
    if (curlabel->justify & LATEXLABEL) return False;
 
-   cfont = findcurfont(textpos, curlabel->string, areawin->topinstance);
+   cfont = findcurfont(areawin->textpos, curlabel->string, areawin->topinstance);
    composefontlib(cfont);
    startcatalog(NULL, FONTLIB, NULL);
    return True;
