@@ -49,7 +49,6 @@ extern XCWindowData *areawin;
 #ifndef TCL_WRAPPER
 extern Widget     menuwidgets[];
 #endif
-extern short textpos, textend;
 extern char _STR[150];
 
 /*----------------------------------------------------------------------*/
@@ -1835,7 +1834,7 @@ void makeparam(labelptr thislabel, char *key)
 
    if (paramcross(topobject, thislabel)) {
       Wprintf("Parameters cannot be nested!");
-      textend = 0;
+      areawin->textend = 0;
       return;
    }
 
@@ -1847,16 +1846,17 @@ void makeparam(labelptr thislabel, char *key)
    /* First, place PARAM_START and PARAM_END structures at the	*/
    /* intended parameter boundaries				*/
 
-   if (textend > 0 && textend < textpos) {  	/* partial string */
-      splitstring(textend, &thislabel->string, areawin->topinstance);
-      splitstring(textpos, &thislabel->string, areawin->topinstance);
+   if (areawin->textend > 0 && areawin->textend < areawin->textpos) {
+      /* partial string */
+      splitstring(areawin->textend, &thislabel->string, areawin->topinstance);
+      splitstring(areawin->textpos, &thislabel->string, areawin->topinstance);
 
       /* Because "splitstring" changes all the pointers, find the    */
       /* stringpart structures at textend and textpos positions.     */
 
-      begpart = findstringpart(textend, NULL, thislabel->string,
+      begpart = findstringpart(areawin->textend, NULL, thislabel->string,
 		areawin->topinstance);
-      endpart = findstringpart(textpos, NULL, thislabel->string,
+      endpart = findstringpart(areawin->textpos, NULL, thislabel->string,
 		areawin->topinstance);
 
       /* Make the new segments for PARAM_START and PARAM_END.	*/
@@ -1886,7 +1886,7 @@ void makeparam(labelptr thislabel, char *key)
    begpart->nextpart = endpart->nextpart;
    endpart->nextpart = NULL;
 
-   textend = 0;
+   areawin->textend = 0;
    incr_changes(topobject);
    if (newkey != key) free(newkey);
 }
@@ -2112,11 +2112,12 @@ void unparameterize(int mode)
    else
       ptype = ALL_TYPES;
 
-   if ((areawin->selects == 1) && (mode == P_SUBSTRING) && textend > 0
-		&& textend < textpos) {
+   if ((areawin->selects == 1) && (mode == P_SUBSTRING) && areawin->textend > 0
+		&& areawin->textend < areawin->textpos) {
       if (SELECTTYPE(areawin->selectlist) != LABEL) return;	 /* Not a label */
       settext = SELTOLABEL(areawin->selectlist);
-      strptr = findstringpart(textend, &locpos, settext->string, areawin->topinstance);
+      strptr = findstringpart(areawin->textend, &locpos, settext->string,
+		areawin->topinstance);
       while (strptr != NULL && strptr->type != PARAM_END)
 	 strptr = strptr->nextpart;
       if (strptr == NULL) return;	/* No parameters */
@@ -2200,15 +2201,16 @@ Boolean paramcross(objectptr tobj, labelptr tlab)
    stringpart *firstptr, *lastptr;
    int locpos;
 
-   lastptr = findstringpart(textpos, &locpos, tlab->string, areawin->topinstance);
+   lastptr = findstringpart(areawin->textpos, &locpos, tlab->string,
+			areawin->topinstance);
 
    /* This text position can't be inside another parameter */
    for (firstptr = lastptr; firstptr != NULL; firstptr = firstptr->nextpart)
       if (firstptr->type == PARAM_END) return True;
 
    /* The area between textend and textpos cannot contain a parameter */
-   if (textend > 0)
-      for (firstptr = findstringpart(textend, &locpos, tlab->string,
+   if (areawin->textend > 0)
+      for (firstptr = findstringpart(areawin->textend, &locpos, tlab->string,
 		areawin->topinstance); firstptr != lastptr;
 		firstptr = firstptr->nextpart)
          if (firstptr->type == PARAM_START || firstptr->type == PARAM_END)
