@@ -47,7 +47,6 @@ extern Cursor	appcursors[NUM_CURSORS];
 extern XCWindowData *areawin;
 extern Globaldata xobjs;
 extern char _STR[150];
-extern short textpos, textend;
 
 #ifdef TCL_WRAPPER 
 extern Tcl_Interp *xcinterp;
@@ -396,12 +395,6 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 
    for (curgen = selobj->plist; curgen < selobj->plist + selobj->parts; curgen++) {
 
-      /* when selection element to attach to, don't select the element */
-      /* to be attached. . .					       */
-
-      if (areawin->attachto == TRUE && eventmode != COPY_MODE && eventmode !=
-	    MOVE_MODE && curgen == EDITPART) continue;
-
       selected = False;
 
       /* Check among polygons, arcs, and curves */
@@ -434,7 +427,7 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 
 	    selected = test_insideness(areawin->save.x, areawin->save.y,
 		newboxpts);
-	    if (selected) textpos = textend = 0;
+	    if (selected) areawin->textpos = areawin->textend = 0;
 	 }
       }
 
@@ -457,13 +450,23 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 	 genericptr *pathp;
 
 	 /* Accept path if any subcomponent of the path is accepted */
-	 /* Record which part was selected in the "subeditpart" variable */
+	 /* Record which part was selected by setting its cycle to zero */
 
  	 for (pathp = TOPATH(curgen)->plist; pathp < TOPATH(curgen)->plist +
 		TOPATH(curgen)->parts; pathp++)
 	    if (pathselect(pathp, SPLINE|ARC|POLYGON, range)) {
 	       selected = True;
-	       areawin->editsubpart = (short)(pathp - TOPATH(curgen)->plist);
+	       switch((*pathp)->type) {
+	          case POLYGON:
+		     TOPOLY(pathp)->cycle = 0;
+		     break;
+	          case ARC:
+		     TOARC(pathp)->cycle = 0;
+		     break;
+	          case SPLINE:
+		     TOSPLINE(pathp)->cycle = 0;
+		     break;
+	       }
 	       break;
 	    }
       }
@@ -583,7 +586,7 @@ void selectarea()
       areawin->save.x = tmp;
    }
 
-   textpos = textend = 0;
+   areawin->textpos = areawin->textend = 0;
    for (curgen = topobject->plist; curgen < topobject->plist
 	+ topobject->parts; curgen++) {
 
@@ -660,20 +663,20 @@ void selectarea()
 		 if (isect) {
 		    state = 1;
 		    selected = True;
-		    textend = j;
-		    if ((textend > 1) && strptr->type != TEXT_STRING)
-		       textend--;
+		    areawin->textend = j;
+		    if ((areawin->textend > 1) && strptr->type != TEXT_STRING)
+		       areawin->textend--;
 		 }
 	      }
 	      else {
 		 if (!isect) {
-		    textpos = j;
+		    areawin->textpos = j;
 		    state = 2;
 		    break;
 		 }
 	      }
 	   }
-	   if (state == 1) textpos = j;   /* selection goes to end of string */
+	   if (state == 1) areawin->textpos = j;   /* selection goes to end of string */
 	   } break;
 	   
 	case(PATH):
@@ -712,7 +715,7 @@ void selectarea()
 
    if (!checkselect(LABEL) || areawin->selects != 1 ||
 	(areawin->selects == 1 && SELECTTYPE(areawin->selectlist) != LABEL)) {
-      textpos = textend = 0;
+      areawin->textpos = areawin->textend = 0;
    }
 
    /* Register the selection as an undo event */
@@ -815,6 +818,7 @@ void freeselects()
 void clearselects_noundo()
 {
    if (areawin->selects > 0) {
+      reset_connected_pins();
       freeselects();
       if (xobjs.suspend < 0) {
          setallstylemarks(areawin->style);
@@ -1073,6 +1077,155 @@ Boolean compareselection(selection *sa, selection *sb)
    return (match == sa->selects) ? True : False;
 }
 
+/*----------------------------------------------------------------------*/
+/* Add pin cycles connected to selected labels				*/
+/*----------------------------------------------------------------------*/
+
+void label_connect_cycles(labelptr thislab)
+{
+   genericptr *pgen;
+   Boolean is_selected;
+   XPoint *testpt;
+   polyptr cpoly;
+   short *stest, cycle;
+
+   if (thislab->pin == LOCAL || thislab->pin == GLOBAL) {
+      for (pgen = topobject->plist; pgen < topobject->plist +
+			topobject->parts; pgen++) {
+	 /* Ignore any wires that are already selected */
+	 is_selected = FALSE;
+	 for (stest = areawin->selectlist; stest < areawin->selectlist +
+			areawin->selects; stest++) {
+	    if (SELTOGENERIC(stest) == *pgen) {
+	       is_selected = TRUE;
+	       break;
+	    }
+	 }
+	 if (ELEMENTTYPE(*pgen) == POLYGON) {
+	    cpoly = TOPOLY(pgen);
+	    if (!is_selected) {
+	       cycle = 0;
+	       for (testpt = cpoly->points; testpt < cpoly->points +
+			cpoly->number; testpt++) {
+		  if (testpt->x == thislab->position.x
+			&& testpt->y == thislab->position.y) {
+		     cpoly->cycle = cycle;
+		     break;
+		  }
+		  else
+		     cycle++;
+	       }
+	    }
+	    else {
+	       /* Make sure that this polygon's cycle is not set! */
+	       cpoly->cycle = -1;
+	    }
+	 }
+      }
+   }
+}
+
+/*----------------------------------------------------------------------*/
+/* Add pin cycles connected to selected instances			*/
+/*----------------------------------------------------------------------*/
+
+void inst_connect_cycles(objinstptr thisinst)
+{
+   genericptr *ggen, *pgen;
+   Boolean is_selected;
+   XPoint refpoint, *testpt;
+   labelptr clab;
+   polyptr cpoly;
+   short *stest, cycle;
+   objectptr thisobj = thisinst->thisobject;
+
+   for (ggen = thisobj->plist; ggen < thisobj->plist + thisobj->parts; ggen++) {
+      if (ELEMENTTYPE(*ggen) == LABEL) {
+	 clab = TOLABEL(ggen);
+	 if (clab->pin == LOCAL || clab->pin == GLOBAL) {
+	    ReferencePosition(thisinst, &clab->position, &refpoint);
+	    for (pgen = topobject->plist; pgen < topobject->plist +
+			topobject->parts; pgen++) {
+	       /* Ignore any wires that are already selected */
+	       is_selected = FALSE;
+	       for (stest = areawin->selectlist; stest < areawin->selectlist +
+			areawin->selects; stest++) {
+		  if (SELTOGENERIC(stest) == *pgen) {
+		     is_selected = TRUE;
+		     break;
+		  }
+	       }
+	       if (ELEMENTTYPE(*pgen) == POLYGON) {
+		  cpoly = TOPOLY(pgen);
+	          if (!is_selected) {
+		     cycle = 0;
+		     for (testpt = cpoly->points; testpt < cpoly->points +
+				cpoly->number; testpt++) {
+		        if (testpt->x == refpoint.x && testpt->y == refpoint.y) {
+			   cpoly->cycle = cycle;
+			   break;
+		        }
+		        else
+		           cycle++;
+		     }
+		  }
+	          else {
+		     /* Make sure that this polygon's cycle is not set! */
+		     cpoly->cycle = -1;
+		  }
+	       }
+	    }
+	 }
+      }
+   }
+}
+
+/*----------------------------------------------------------------------*/
+/* Select connected pins on all selected object instances and labels	*/
+/*----------------------------------------------------------------------*/
+
+void select_connected_pins()
+{
+   short *selptr;
+   objinstptr selinst;
+   labelptr sellab;
+
+   if (!areawin->pinattach) return;
+
+   for (selptr = areawin->selectlist; selptr < areawin->selectlist +
+		areawin->selects; selptr++) {
+      switch (SELECTTYPE(selptr)) {
+	 case LABEL:
+            sellab = SELTOLABEL(selptr);
+            label_connect_cycles(sellab);
+	    break;
+	 case OBJINST:
+            selinst = SELTOOBJINST(selptr);
+            inst_connect_cycles(selinst);
+	    break;
+      }
+   }
+}
+
+/*----------------------------------------------------------------------*/
+/* Reset all polygon cycles flagged during a move (polygon wires	*/
+/* connected to pins of an object instance).				*/
+/*----------------------------------------------------------------------*/
+
+void reset_connected_pins()
+{
+   polyptr cpoly;
+   genericptr *pgen;
+
+   for (pgen = topobject->plist; pgen < topobject->plist +
+			topobject->parts; pgen++) {
+      if (ELEMENTTYPE(*pgen) == POLYGON) {
+	 cpoly = TOPOLY(pgen);
+	 cpoly->cycle = -1;
+      }
+   }
+}
+
 /*----------------------------------------------------------------------*/
 /* Recursive selection mechanism					*/
 /*----------------------------------------------------------------------*/
