@@ -35,7 +35,6 @@
 extern Display *dpy;
 extern XCWindowData *areawin;
 extern Globaldata xobjs;
-extern short textpos, textend;
 extern short fontcount;
 extern fontinfo *fonts;
 extern int *appcolors;
@@ -1601,9 +1600,9 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
 
 		   scolor = curcolor;
 	           if (passcolor == DOSUBSTRING) {
-	              if (pos <= textpos && pos > textend)
+	              if (pos <= areawin->textpos && pos > areawin->textend)
 		         scolor = SELECTCOLOR;
-		      else if (pos > textpos) {
+		      else if (pos > areawin->textpos) {
       		         XTopSetForeground(curcolor);
 		      }
 	           }
@@ -1692,14 +1691,18 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
 	    xtotal = 0.5;
 	    break;
 	 case HALFSPACE: 
-	    chptr = (*(somebet + 32));
-	    xtotal += (float)(chptr->bbox.width + chptr->bbox.lowerleft.x)
+	    if (somebet) {
+	       chptr = (*(somebet + 32));
+	       xtotal += (float)(chptr->bbox.width + chptr->bbox.lowerleft.x)
 			* locscale * natscale / 2;
+	    }
 	    break;
 	 case QTRSPACE:
-	    chptr = (*(somebet + 32));
-	    xtotal += (float)(chptr->bbox.width + chptr->bbox.lowerleft.x) 
+	    if (somebet) {
+	       chptr = (*(somebet + 32));
+	       xtotal += (float)(chptr->bbox.width + chptr->bbox.lowerleft.x) 
 			* locscale * natscale / 4;
+	    }
 	    break;
 	 case TABBACKWARD:	/* find first tab value with x < xtotal */
 	    for (tabno = numtabs - 1; tabno >= 0; tabno--) {
@@ -1735,10 +1738,12 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
 	    ykern += strptr->data.kern[1];
 	    break;
 	 case FONT_NAME:
-	    somebet = fonts[strptr->data.font].encoding;
-	    locscale = fonts[strptr->data.font].scale;
-	    if (ykern == 0.0)
-	       natscale = locscale;
+	    if (strptr->data.font < fontcount) {
+	       somebet = fonts[strptr->data.font].encoding;
+	       locscale = fonts[strptr->data.font].scale;
+	       if (ykern == 0.0)
+	          natscale = locscale;
+	    }
 	    break;
 	 case TEXT_STRING:
 	    textptr = strptr->data.string;
@@ -1756,6 +1761,7 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
 		}
 	    }
 
+	    if (somebet == NULL) break;
 	    for (; textptr && *textptr != '\0'; textptr++) {
                if (dostop && (locpos >= dostop)) break;
 	       locpos++;
