@@ -120,7 +120,6 @@ fontinfo *fonts;
 /* Externally defined variables						*/
 /*----------------------------------------------------------------------*/
 
-extern short textpos;
 extern aliasptr aliastop;
 extern float version;
 
@@ -1060,10 +1059,11 @@ XCWindowData *create_new_window()
    newwindow->hierstack = NULL;
    initmem(newwindow->editstack);
    newwindow->pinpointon = False;
+   newwindow->pinattach = False;
    newwindow->buschar = '(';	/* Vector notation for buses */
    newwindow->defaultcursor = &CROSS;
    newwindow->event_mode = NORMAL_MODE;
-   newwindow->attachto = FALSE;
+   newwindow->attachto = -1;
    newwindow->color = DEFAULTCOLOR;
    newwindow->gccolor = 0;
    newwindow->gctype = 0;
@@ -1239,7 +1239,6 @@ nexthier:
             Tcl_ResetResult(interp);
             Tcl_AppendToObj(Tcl_GetObjResult(interp), 
 		"handle is identified by leading H and hexidecimal value only", -1);
-            TclCheckBadOctal(interp, string);
         }
         free_stack(&newstack);
         return TCL_ERROR;
@@ -1258,7 +1257,6 @@ nexthier:
             sprintf(buf, "expected handle but got \"%.50s\"", string);
             Tcl_ResetResult(interp);
             Tcl_AppendToObj(Tcl_GetObjResult(interp), buf, -1);
-            TclCheckBadOctal(interp, string);
         }
         free_stack(&newstack);
         return TCL_ERROR;
