@@ -1364,6 +1364,8 @@ typedef struct _windowdata {
    short	editpart;
    short	attachto;
    short	lastlibrary;
+   short	textpos;
+   short	textend;
    objinstptr	topinstance;
    objectptr	editstack;
    Matrixptr	MatStack;
