@@ -53,7 +53,6 @@ extern ApplicationData appdata;
 extern Cursor	  appcursors[NUM_CURSORS];
 extern fontinfo *fonts;
 extern short fontcount;
-extern short textpos, textend;
 extern xcWidget    menuwidgets[];
 extern xcWidget	 toolbar;
 extern short	 popups;	     /* total number of popup windows */
@@ -418,8 +417,10 @@ void getkern(xcWidget button, caddr_t nulldata, caddr_t calldata)
 
    if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       labelptr curlabel = TOLABEL(EDITPART);
-      strptr = findstringpart(textpos - 1, NULL, curlabel->string, areawin->topinstance);
-      nextptr = findstringpart(textpos, NULL, curlabel->string, areawin->topinstance);
+      strptr = findstringpart(areawin->textpos - 1, NULL, curlabel->string,
+			areawin->topinstance);
+      nextptr = findstringpart(areawin->textpos, NULL, curlabel->string,
+			areawin->topinstance);
       if (strptr->type == KERN) {
 	 kx = strptr->data.kern[0];
 	 ky = strptr->data.kern[1];
@@ -1150,36 +1151,6 @@ void renamepage(short pagenumber)
    }
    else if (button == NULL)
       Fprintf(stderr, "Error:  No Button Widget named \"%9s\"\n", bname);
-
-   /* The GUI has been taken care of, but the page itself might not exist */
-   if (!thisinst) return;
-
-   /* If the page had a master but now has a different	*/
-   /* name, disconnect the schematics.			*/
-
-   if (thisinst->thisobject->schemtype == SECONDARY) {
-      if (strcmp(thisinst->thisobject->name, thisinst->thisobject->symschem->name)) {
-	 thisinst->thisobject->symschem = NULL;
-	 thisinst->thisobject->schemtype = PRIMARY;
-      }
-      Wprintf("Page disconnected from master schematic.");
-   }
-
-   /* Conversely, if the page has the same name as another page, the	*/
-   /* schematics are considered connected.				*/
-
-   for (page = 0; page < xobjs.pages; page++) {
-      objinstptr pageinst;
-      if (page == pagenumber) continue;
-      pageinst = xobjs.pagelist[page]->pageinst;
-      if (pageinst && !strcmp(pageinst->thisobject->name,
-		thisinst->thisobject->name)) {
-	 thisinst->thisobject->symschem = pageinst->thisobject;
-	 thisinst->thisobject->schemtype = SECONDARY;
-         Wprintf("Page connected to master schematic.");
-	 break;
-      }
-   }
 }
 
 /*--------------------------------------------------------------*/
@@ -1364,8 +1335,10 @@ void setcolor(xcWidget w, pointertype value, caddr_t calldata)
 
    if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       labelptr curlabel = TOLABEL(EDITPART);
-      strptr = findstringpart(textpos - 1, NULL, curlabel->string, areawin->topinstance);
-      nextptr = findstringpart(textpos, NULL, curlabel->string, areawin->topinstance);
+      strptr = findstringpart(areawin->textpos - 1, NULL, curlabel->string,
+			areawin->topinstance);
+      nextptr = findstringpart(areawin->textpos, NULL, curlabel->string,
+			areawin->topinstance);
       if (strptr->type == FONT_COLOR) {
 	 undrawtext(curlabel);
 	 strptr->data.color = cindex;
