@@ -17,6 +17,7 @@
 #include <errno.h>
 #include <limits.h>
 #include <locale.h>
+#ifndef XC_WIN32
 #include <unistd.h>   /* for unlink() */
 
 #include <X11/Intrinsic.h>
@@ -38,6 +39,7 @@
 #include "Xw/BBoard.h"
 #include "Xw/TextEdit.h"
 #include "Xw/Toggle.h"
+#endif
 
 /*----------------------------------------------------------------------*/
 /* Local includes							*/
@@ -54,7 +56,9 @@
 #include "prototypes.h"
 
 #ifdef HAVE_XPM
+#ifndef XC_WIN32
 #include <X11/xpm.h>
+#endif
 #include "lib/pixmaps/xcircuit.xpm"
 #endif
 
@@ -65,7 +69,8 @@
 short	 popups;      /* total number of popup widgets on the screen */
 Boolean	 pressmode;   /* Whether we are in a press & hold state */
 
-xcWidget   message1, message2, message3, top, toolbar, overlay = NULL;
+xcWidget   top;
+xcWidget   message1, message2, message3, toolbar, overlay = NULL;
 xcWidget   menuwidgets[MaxMenuWidgets];
 xcWidget   wschema, wsymb, netbutton;
 XtAppContext app;
@@ -88,7 +93,6 @@ extern xcIntervalId printtime_id;
 extern short beeper;
 extern short fontcount;
 extern fontinfo *fonts;
-extern short textpos;
 extern short help_up;
 extern menustruct TopButtons[];
 #ifdef HAVE_XPM
@@ -132,6 +136,8 @@ XContext XUniqueContext()
 /* Initial Resource Management						*/
 /*----------------------------------------------------------------------*/
 
+#ifndef XC_WIN32
+
 static XtResource resources[] = {
 
   /* schematic layout colors */
@@ -221,6 +227,8 @@ static XtResource resources[] = {
       XtOffset(ApplicationDataPtr, timeout), XtRString, "10"}
 };
 
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Add a new color button to the color menu				*/
 /* 	called if new color button needs to be made 			*/
@@ -341,6 +349,8 @@ void makesubmenu(char *menuname, char *attachname, menuptr buttonmenu,
 /* Toolbar Creator							*/
 /*----------------------------------------------------------------------*/
 
+#ifndef XC_WIN32
+
 void createtoolbar (xcWidget abform, Widget aform)
 {
    int i, n = 0;
@@ -386,6 +396,8 @@ void createtoolbar (xcWidget abform, Widget aform)
    }
 }
 
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Toolbar Resize							*/
 /*----------------------------------------------------------------------*/
@@ -566,6 +578,8 @@ void quitcheck(xcWidget w, caddr_t clientdata, caddr_t calldata)
 /* Propogate any key event from the dialog box into the textedit widget    */
 /*-------------------------------------------------------------------------*/
 
+#ifndef XC_WIN32
+
 void propevent(xcWidget w, xcWidget editwidget, XEvent *event)
 {
    Window ewin = xcWindow(editwidget);
@@ -574,6 +588,8 @@ void propevent(xcWidget w, xcWidget editwidget, XEvent *event)
    XSendEvent(dpy, ewin, False, KeyPressMask, event);
 }
 
+#endif
+
 /*-------------------------------------------------------------------------*/
 /* Destroy an interactive text-editing popup box 			   */
 /*-------------------------------------------------------------------------*/
@@ -644,6 +660,8 @@ void linkset(xcWidget button, propstruct *callstruct, caddr_t calldata)
 /* Pull text from the popup prompt buffer into a global string variable */
 /*----------------------------------------------------------------------*/
 
+#ifndef XC_WIN32
+
 void xcgettext(xcWidget button, popupstruct *callstruct, caddr_t calldata)
 {
    if (callstruct->textw != NULL) {
@@ -678,6 +696,8 @@ void xcgettext(xcWidget button, popupstruct *callstruct, caddr_t calldata)
    destroypopup(button, callstruct, calldata);
 }
 
+#endif
+
 /*-------------------------------------------------------------------------*/
 /* Grab text from the "output properties" window			   */
 /*-------------------------------------------------------------------------*/
@@ -697,7 +717,7 @@ void getproptext(xcWidget button, propstruct *callstruct, caddr_t calldata)
    /* special stuff for filename changes */
 
    if (file_yes) {
-      char blabel[50];
+      char blabel[1024];
       short num_linked;
       xcWidget wrbutton, ltoggle;
       struct stat statbuf;
@@ -810,6 +830,8 @@ void updatename(xcWidget button, xcWidgetList callstruct, caddr_t calldata)
 /* and text and label fields.						   */
 /*-------------------------------------------------------------------------*/
 
+#ifndef XC_WIN32
+
 void popupprompt(xcWidget button, char *request, char *current, void (*function)(),
 	buttonsave *datastruct, const char *filter)
 {
@@ -934,6 +956,8 @@ void popupprompt(xcWidget button, char *request, char *current, void (*function)
 	TEXTPTR);
 }
 
+#endif
+
 /*-------------------------------------------------------------------------*/
 /* Create a popup window for property changes				   */
 /*-------------------------------------------------------------------------*/
@@ -943,6 +967,8 @@ void popupprompt(xcWidget button, char *request, char *current, void (*function)
 
 propstruct okstruct[MAXPROPS], fpokstruct;
 
+#ifndef XC_WIN32
+
 void outputpopup(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
    buttonsave  *savebutton;
@@ -1267,13 +1293,15 @@ void outputpopup(xcWidget button, caddr_t clientdata, caddr_t calldata)
       XDefineCursor(dpy, xcWindow(entertext[i]), TEXTPTR);
 }
 
+#endif
+
 /*-------------------------------------------------*/
 /* Print a string to the message widget. 	   */ 
 /* Note: Widget message must be a global variable. */
 /* For formatted strings, format first into _STR   */
 /*-------------------------------------------------*/
 
-xcTimeOutProc clrmessage(caddr_t clientdata, xcIntervalId *id)
+void clrmessage(XtPointer clientdata, xcIntervalId *id)
 {
    char buf1[50], buf2[50];
 
@@ -1338,7 +1366,7 @@ void xc_vprintf(xcWidget widget, const char *fmt, va_list args_in)
 
    /* 10 second timeout */
    if (widget == message3) {
-      printtime_id = xcAddTimeOut(app, 10000, (xcTimeOutProc)clrmessage, NULL);
+      printtime_id = xcAddTimeOut(app, 10000, clrmessage, NULL);
    }
 }
     
@@ -1391,6 +1419,8 @@ void W2printf(char *format, ...)
 
 /*------------------------------------------------------------------------------*/
 
+#ifndef XC_WIN32
+
 void getcommand(xcWidget cmdw, caddr_t clientdata, caddr_t calldata)
 {
    sprintf(_STR2, "%.249s", XwTextCopyBuffer(cmdw));
@@ -1485,6 +1515,8 @@ int installowncmap()
    return(1);
 }
 
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Event handler for input focus					*/
 /*----------------------------------------------------------------------*/
@@ -1551,6 +1583,12 @@ void delwin(xcWidget w, popupstruct *bstruct, XClientMessageEvent *event)
 /* callback functions and event handlers.				*/
 /*----------------------------------------------------------------------*/
 
+#ifdef XC_WIN32
+
+XCWindowData* GUI_init(int argc, char *argv[]);
+
+#else
+
 XCWindowData *GUI_init(int argc, char *argv[])
 {
 
@@ -1566,7 +1604,10 @@ XCWindowData *GUI_init(int argc, char *argv[])
    XCWindowData *newwin;
    short i, n = 0;
 
-   char *argfb[] = {			/* Fallback resources */
+   /* Things to do the first time around */
+   if (dpy == NULL) {
+      
+      char *argfb[] = {			/* Fallback resources */
 	"xcircuit*foreground : brown4", /* These are the values that 	*/
 	"xcircuit*background : beige",  /* not or cannot be explicitly	*/
 	"xcircuit.foreground : black",	/* initialized by		*/
@@ -1574,59 +1615,66 @@ XCWindowData *GUI_init(int argc, char *argv[])
 	"xcircuit*borderWidth : 2",	/* below.			*/
 	"xcircuit*borderColor : Red",
 	NULL				/* Sentinel */
-   };
+      };
 
-   XtSetLanguageProc(NULL, NULL, NULL);
+      XtSetLanguageProc(NULL, NULL, NULL);
 
-   /*-------------------------------------------------------------------*/
-   /* Set pointer to own XDefaults file, but allow an external override */
-   /*-------------------------------------------------------------------*/
+      /*-------------------------------------------------------------------*/
+      /* Set pointer to own XDefaults file, but allow an external override */
+      /*-------------------------------------------------------------------*/
 
 #ifdef HAVE_PUTENV
-   if (getenv("XAPPLRESDIR") == NULL)
-      putenv("XAPPLRESDIR=" RESOURCES_DIR);
+      if (getenv("XAPPLRESDIR") == NULL)
+         putenv("XAPPLRESDIR=" RESOURCES_DIR);
 #else
-   setenv("XAPPLRESDIR", RESOURCES_DIR, 0);
+      setenv("XAPPLRESDIR", RESOURCES_DIR, 0);
 #endif
 
-   /*-----------------------------*/
-   /* Create the widget hierarchy */
-   /*-----------------------------*/
+      /*-----------------------------*/
+      /* Create the widget hierarchy */
+      /*-----------------------------*/
 
-   top = XtOpenApplication(&app, "XCircuit", NULL, 0, &argc, argv,
+      top = XtOpenApplication(&app, "XCircuit", NULL, 0, &argc, argv,
 		argfb, applicationShellWidgetClass, NULL, 0);
 
-   dpy = XtDisplay(top);
-   win = DefaultRootWindow(dpy);
-   cmap = DefaultColormap(dpy, DefaultScreen(dpy));
+      dpy = XtDisplay(top);
+      win = DefaultRootWindow(dpy);
+      cmap = DefaultColormap(dpy, DefaultScreen(dpy));
 
-   /*-------------------------*/
-   /* Create stipple patterns */
-   /*-------------------------*/
+      /*-------------------------*/
+      /* Create stipple patterns */
+      /*-------------------------*/
 
-   for (i = 0; i < STIPPLES; i++)
-      STIPPLE[i] = XCreateBitmapFromData(dpy, win, STIPDATA[i], 4, 4);
+      for (i = 0; i < STIPPLES; i++)
+         STIPPLE[i] = XCreateBitmapFromData(dpy, win, STIPDATA[i], 4, 4);
 
-   /*----------------------------------------*/
-   /* Allocate space for the basic color map */
-   /*----------------------------------------*/
+      /*----------------------------------------*/
+      /* Allocate space for the basic color map */
+      /*----------------------------------------*/
 
-   number_colors = 0;
-   colorlist = (colorindex *)malloc(sizeof(colorindex));
-   appcolors = (int *) malloc(NUMBER_OF_COLORS * sizeof(int));
+      number_colors = 0;
+      colorlist = (colorindex *)malloc(sizeof(colorindex));
+      appcolors = (int *) malloc(NUMBER_OF_COLORS * sizeof(int));
 
-   /*-----------------------------------------------------------*/
-   /* Xw must add these translations for the popup manager      */
-   /*-----------------------------------------------------------*/
-   XwAppInitialize(app);
+      /*-----------------------------------------------------------*/
+      /* Xw must add these translations for the popup manager      */
+      /*-----------------------------------------------------------*/
+      XwAppInitialize(app);
 
-   /*-------------------------------*/
-   /* Get the application resources */
-   /*-------------------------------*/
+      /*-------------------------------*/
+      /* Get the application resources */
+      /*-------------------------------*/
 
-   XtAppAddConverter(app, XtRString, XtRPixel, (XtConverter)CvtStringToPixel, NULL, 0);
-   XtGetApplicationResources(top, &appdata, resources, XtNumber(resources), 
-	NULL, 0);
+      XtAppAddConverter(app, XtRString, XtRPixel, (XtConverter)CvtStringToPixel,
+		NULL, 0);
+      XtGetApplicationResources(top, &appdata, resources, XtNumber(resources), 
+		NULL, 0);
+   }
+   else {	/* Not the first time---display has already been opened */
+
+      top = XtAppCreateShell("XCircuit", "XCircuit", applicationShellWidgetClass,
+		dpy, NULL, 0);
+   }
 
    n = 0;
    XtnSetArg(XtNwidth, appdata.width);
@@ -1638,7 +1686,7 @@ XCWindowData *GUI_init(int argc, char *argv[])
 
    form = XtCreateManagedWidget("Form", XwformWidgetClass, top, NULL, 0);
 
-   /* Generate a new window */
+   /* Generate a new window data structure */
 
    newwin = create_new_window();
 
@@ -1934,6 +1982,8 @@ XCWindowData *GUI_init(int argc, char *argv[])
    return newwin;
 }
 
+#endif
+
 /*----------------------------------------------------------------------*/
 /* When not using ToolScript, this is the standard X loop (XtMainLoop())*/
 /*----------------------------------------------------------------------*/
